; EXAMPLE 2: call INSIDER, W=parameter
;
; Podprogram vypise obsah registru W na displej Insideru.
; V datove pameti se spotrebuje 1 registr (IREG), spotrebuje se
; 1 uroven STACKU. Hodnota W zustane nezmenena. Programova narocnost
; procedury je 20 instrukci.
;-----------------------------------------------------------------------
;       DEFINICE HARDWARE
        list    F=inhx8m, p=16F84, r=hex        ; pro prekladac
        __config        3ffBh                   ; pojistky
;-----------------------------------------------------------------------
;       User Pins                               Tris: 3F
#define     ICLK    portb,7
#define     IDATA   portb,6
;       Special Function Registers
INDF        equ     00h     ; sdileny pro obe banky
TMR0        equ     01h
ROption     equ     01h
PCL         equ     02h     ; sdileny pro obe banky
STATUS      equ     03h     ; sdileny pro obe banky
FSR         equ     04h     ; sdileny pro obe banky
Porta       equ     05h     ; bank 0
Portb       equ     06h     ; bank 0
Trisa       equ     05h     ; bank 1
Trisb       equ     06h     ; bank 1

#define zero    status,2    ; Zero flag
#define DC      status,1    ; DC flag
#define carry   status,0    ; C flag

;       Registry Insideru
IREG     set         0x0C   ; pracovni registr Insideru

;       Uzivatelske registry
;UserReg set         0x0D   ; zde jsou uzivatelske registry...
;-----------------------------------------------------------------------
;       Definice maker
bnk0    macro               ; makro nastavi banku 0
        bcf     status,5
        endm
bnk1    macro               ; makro nastavi banku 1
        bsf     status,5
        endm
;-----------------------------------------------------------------------
;       Startup
        org     0x00
        goto    Start       ; 1. instrukce - skok na zacatek programu
;-----------------------------------------------------------------------
;       Procedury
        org     0x05        ; Od teto adresy bude ulozen program
Insider:                    ; Procedura vypise nedestruktivne W na displeji
                            ; insideru, vsechny prepinace Insideru jsou
                            ; nastaveny do 0 (CP=0,EU=0,EF=0,EP=0)
        movwf   IREG        ; Ulozeni W do IREG
        movlw   0x08        ; pocatecni inicializace cyklu

ICykl:  rlf     IREG,f
        bcf     IDATA
        btfsc   carry       ; test bitu a nastaveni pinu DATA
        bsf     IDATA
        bsf     ICLK        ; zapis sestupnou hranou
        bcf     ICLK

        btfss   carry       ; test na stav carry
        goto    Ic0         ; carry je 0
        addlw   -1          ; carry byl 1, W=W-1
        bsf     carry       ; obnoveni stavu carry
Istop:  btfss   zero
        goto    Icykl       ; opakuje se 8x

        rlf     IREG,f      ; obnoveni stavu registru IREG
        movf    IREG,w      ; obnoveni registru W
        return
Ic0:    addlw   -1          ; carry byl 0, W=W-1
        bcf     carry       ; obnoveni stavu carry
        goto    Istop
;-----------------------------------------------------------------------
Start:  bnk1
        movlw   0x3F
        movwf   TRISB       ; nastaveni smeru portu B
        bnk0                ; (pin 6,7 output, pin 0 az 5 input)

        nop
        nop                 ; ...kdekoliv v programu

        movlw   0x1A
        call    Insider     ; nedestruktivne vypise W na displeji

        nop
        nop                 ; ...kdekoliv v programu

        addlw   0x01        ; inkrementujeme W
        call    Insider     ; a opet vypiseme na displeji

        nop
        nop                 ; ...uzivatelsky program pokracuje
Stop:   goto Stop
        end
